/*
# Projeto:
	Demonstração de controle de servo motor usando Displays (LCM) Proculus.

# Descrição:
	Exemplo de controle de um servo motor usando um Display Proculus (UnicView AD).
	A posição do servo é controlada pelo Arduino, e definida através de um destes métodos:

	Botões e sliders no Display.
	Potenciômetro

	No Display, a posição pode ser atualizada deslizando um slider, ou tocando nos botões
	com posições predefinidas.

# Documentação do Projeto:
	http://www.victorvision.com.br/projects/arduino-system/library/site/examples/sensors/

# Autores:
	Renan Candido Vieira
	Andreo de Freitas (revisão)
	Rafael Moura (revisão)

# Licença:
	Copyright (c) 2020 Proculus Technologies Limited and Victor Vision LTDA

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
*/

// *******************************************************************************************************************************************
// * Inclusão das bibliotecas necessárias para o projeto
// *******************************************************************************************************************************************
#include <UnicViewAD.h> 

// *******************************************************************************************************************************************
// * Inicialização do LCM e do Servo
// *******************************************************************************************************************************************
LCM Lcm(Serial);    // Necessário a Inicialização do LCM com o nome de Lcm e como par?metro a Serial que ser? ligado o Display

// *******************************************************************************************************************************************
// * Variáveis Tela de Abertura
// *******************************************************************************************************************************************
const int picIdIntro = 0;         // Endereço da tela de introdução
const int picIdMain = 1;          // Endereço da tela principal
const int return_value = 1000;    // "Return Value" do botão da tela de introdução

// *******************************************************************************************************************************************
// * Variáveis Sensor Ultrassónico
// *******************************************************************************************************************************************
const int trigPin = 3;            // Pino do Arduino a ser usado como trig
const int echopPin = 2;           // Pino do Arduino a ser usado como echo
const int centimetroUnity = 0;    // Valor representativo da unidade centímetro
const int inchUnity = 1;          // Valor representativo da unidade polegadas


bool distUnity;                   // Variável do valor representativo da unidade de medida  que vai ser exibida
bool alarmdist;                   // Variável do alarme de distância
long distMaxCm;                   // Variável de distância selecionada como máxima em centímetro (cm)
long distMinCm;                   // Variável de distância selecionada como mínima em centímetro (cm)
float centimetroDist;             // Variável de distância em centímetro (cm)

// *******************************************************************************************************************************************
// * Variáveis Sensor NTC
// *******************************************************************************************************************************************
const int ntcPin = A1;
const int ntcValue = 10000;           // Valor nominal de 10K do NTC utilizado
const int seriesResistor = 10000;     // Valor nominal de 10K do resistor em serie como o NTC utilizado
const int coefBeta = 3950;            // Valor do coeficiente beta que define relação entre resistência e temperatura de um NTC
const int tempNominal = 25;           // Valor da temperatura nominal (em 25 a resistência é de 10k)
const int sampling = 10;              // Valor de quantas amostras de leitura do analógico serão realizadas
const int analogResolution = 1023;    // Valor de resolução do pino analógico do Arduino utilizado
const int celsiusUnity = false;       // Valor representativo da unidade Celsius
const int fahrenheitUnity = true;     // Valor representativo da unidade Fahrenheit

bool tempUnity;                   // Variável do valor representativo da unidade de medida  que vai ser exibida
bool alarmtemp;                   // Variável do alarme do temperatura
long tempMaxC;                    // Variável da temperatura selecionada como máxima em Celsius (ºC) e Fahrenheit (ºF).
long tempMinC;                    // Variável da temperatura selecionada como mínima em Celsius (ºC) e Fahrenheit (ºF).
float celsiusTemp;                // Variável da temperatura em Celsius (ºC)

// *******************************************************************************************************************************************
// * Variáveis Gerais
// *******************************************************************************************************************************************
const int period = 1000;      // Espaço de tempo entre as leituras da medidas
const int buzzerPin = 10;     // Pino a ser conectado o Buzzer do alarme.

unsigned long timeNow = 0;    // Armazena o tempo absoluto para a próxima leitura dos sensores

// *******************************************************************************************************************************************
// * Inicialização dos objetos LcmVar
// *******************************************************************************************************************************************
LcmVar ExitIntroButton(0);      //  Botão da tela de introdução

LcmVar Distance(100);           // Numeric Display da que indica a distância medida pelo sensor ultrassónico.
LcmVar DistToggleUnity(110);    // Dynamic Icon e Incremental Input que define a unidade de distância a ser exibida entre cent?metro e polegadas.
LcmVar AlarmDistMax(120);       // Numeric Display e Numeric Input que define a distância máxima permitida.
LcmVar AlarmDistMin(130);       // Numeric Display e Numeric Input que define a distância mínima permitida.

LcmVar Temperature(200);        // Numeric Display da que indica a distância medida pelo sensor ultrassónico.
LcmVar TempToggleUnity(210);    // Dynamic Icon e Incremental Input que definem a unidade de distância a ser exibida entre centímetro e polegadas.
LcmVar AlarmTempMax(220);       // Numeric Display e Numeric Input que define a distância máxima permitida.
LcmVar AlarmTempMin(230);       // Numeric Display e Numeric Input que define a distância mínima permitida.

// *******************************************************************************************************************************************
// * Configuração inicial (setup)
// *******************************************************************************************************************************************
void setup()
{
  Lcm.begin();              // Inicia o comunica??o com baud rate de 115200.
  Serial.begin(115200);
  pinMode(trigPin, OUTPUT);       // Declara o "TringPin"  como saída.
  pinMode(echopPin, INPUT);       // Declara o "EchoPin" como entrada.
  pinMode(buzzerPin, OUTPUT);     // Declara o "BuzzerPin" como saída.
  delay(3000);                    // Delay necessário para que o Display inicialize e possa se comunicar
  Lcm.changePicId(picIdIntro);    // Muda para tela de introdução na inicialização do Arduino
}

// *******************************************************************************************************************************************
// * Programa principal (loop)
// *******************************************************************************************************************************************
void loop()
{

  // *******************************************************************************************************************************************
  // * Inicialização da tela principal
  // *
  if (ExitIntroButton.available())                    // Verifica se chegou algo do LcmVar "ExitIntroButton"
  {
    if (ExitIntroButton.getData() == return_value)    // Verifica se o valor recebido pelo  "ExitIntroButton" é o valor definido no botão "return_value"
    {
      DistToggleUnity.write(centimetroUnity);         // O LcmVar "DistToggleUnity" é configurado para centímetro
      AlarmDistMax.write(0);                          // O LcmVar "AlarmDistMax" é zerado
      AlarmDistMin.write(0);                          // O LcmVar "AlarmDistMin" é zerado
      TempToggleUnity.write(celsiusUnity);            // O LcmVar "TempToggleUnity" é configurado para Celsius
      AlarmTempMax.write(0);                          // O LcmVar "AlarmTempMax" é zerado
      AlarmTempMin.write(0);                          // O LcmVar "AlarmTempMin" é zerado
      Lcm.changePicId(picIdMain);                     // Muda da tela de introdução para a tela de main
    }
  }

  // *******************************************************************************************************************************************
  // * Atulização  da leitura dos sensores
  // *
  if (millis() > timeNow + period)                     // Verifica se "millis()" (tempo atual) é maior que a variável "timeNow" (ultima leitura) mais "period" (tempo entre leituras) é tempo da próxima leitura
  {
    float value = 0;                                   // Variável auxiliar que recebe os valores lidos dos senores
    timeNow = millis();                                // Atribui o valor de "millis()"  ao valor da variável "time_now"
    value = Request_dist(distUnity);                   // Atribui o valor de distancia lido no sensor Ultrassónico a variável auxiliar
    Distance.write(value, 2);                          // Escreve a distancia lida no LcmVar "Distance" com  duas casas decimais
    value = Request_temp(tempUnity);                   // Atribui o valor de distancia lido no sensor NTC a variável auxiliar
    Temperature.write(value, 2);                       // Escreve a temperatura lida no LcmVar "Temperature" com  duas casas decimais
    centimetroDist = Request_dist(centimetroUnity);    // Atribui o valor de distancia lida em centímetro a variável centimetroDist
    celsiusTemp = Request_temp(celsiusUnity);          // Atribui o valor de distancia lida em graus Celsius a variável celsiusTemp
  }

  // *******************************************************************************************************************************************
  // * Controle do Alarme de Distancia
  // *
  if ((distMaxCm < centimetroDist) || (distMinCm > centimetroDist))    // Verifica se algum limite máximo ou mínimo foi excedido
  {
    alarmdist = true;
  }
  else
  {
    alarmdist = false;
  }

  // *******************************************************************************************************************************************
  // * Alternância da unidade de medida do sensor de distância
  // *
  if (DistToggleUnity.available())            // Verifica se "DistToggleUnity" foi alternado
  {
    long valueAlarm = 0;                      // Variável auxiliar que recebe os valores máximo e mínimo
    float value = 0;                          // Variável auxiliar que recebe os valores lido do senor      
    distUnity = DistToggleUnity.getData();    // "distUnity" recebe o valor atual escrito no display
    value = Request_dist(distUnity);          // Atribui o valor de distancia lido no sensor Ultrassónico a variável auxiliar
    Distance.write(value, 2);                 // Escreve no LcmVAr "Distance" o valor do sensor de acordo a unidade no display
    if (distUnity == inchUnity)               // Verifica se a unidade é polegadas
    {
      valueAlarm = distMaxCm / 2.54;          // Atribui o valor de distancia máxima a variável auxiliar
      AlarmDistMax.write(valueAlarm);         // Escreve no LcmVAr "AlarmDistMax" o seu valor em polegadas
      valueAlarm = distMinCm / 2.54;          // Atribui o valor de distancia mínima a variável auxiliar
      AlarmDistMin.write(valueAlarm);         // Escreve no LcmVAr "AlarmDistMin" o seu valor em polegadas
    }
    else
    {
      AlarmDistMax.write(distMaxCm);          // Escreve no LcmVAr "AlarmDistMax" o seu valor em centímetro
      AlarmDistMin.write(distMinCm);          // Escreve no LcmVAr "AlarmDistMax" o seu valor em centímetro
    }
  }

  // *******************************************************************************************************************************************
  // * Configuração dos valores mínimo e máximo do sensor de distância
  // *
  if (AlarmDistMax.available())                     // Verifica se o valor máximo foi alterado
  {
    long valueAlarm = 0;                            // Variável auxiliar que recebe o valor máximo
    if (distUnity == inchUnity)                     // Verifica em qual unidade foi escrito o novo valor
    {
      valueAlarm = AlarmDistMax.getData();          // Atribui o valor de distancia máximo a variável auxiliar
      distMaxCm = valueAlarm * 2.54;                // "distMaxCm" recebe o novo valor convertido de polegada para centímetro
    }
    else
    {
      distMaxCm = AlarmDistMax.getData();           // "distMaxCm" recebe o  novo valor máximo em centímetros.
    }
  }
  if (AlarmDistMin.available())                     // Verifica se o valor mínimo foi alterado
  {
    long valueAlarm = 0;                            // Variável auxiliar que recebe o valor mínimo
    if (distUnity == inchUnity)                     // Verifica em qual unidade foi escrito o novo valor
    {
      valueAlarm = AlarmDistMin.getData();          // Verifica em qual unidade foi escrito o novo valor
      distMinCm = AlarmDistMin.getData() * 2.54;    // "distMinCm" recebe o novo valor convertido de polegada para centímetro
    }
    else
    {
      distMinCm = AlarmDistMin.getData();           // "distMinCm" recebe o  novo valor mínimo em centímetros
    }
  }
  // *******************************************************************************************************************************************
  // * Controle do Alarme de temperatura
  // *
  
  if ((tempMaxC < celsiusTemp) || (tempMinC > celsiusTemp)) // Verifica se algum limite máximo ou mínimo foi excedido
  {
    alarmtemp = true;         
  }
  else
  {
    alarmtemp = false;
  }
  // *******************************************************************************************************************************************
  // * Alternância da unidade de medida do sensor de temperatura
  // *
  if (TempToggleUnity.available())                 // Verifica se "TempToggleUnity" foi alternado
  {
    long valueAlarm = 0;                           // Variável auxiliar que recebe os valores máximo e mínimo
    float value = 0;                               // Variável auxiliar que recebe os valores lido do senor      
    tempUnity = TempToggleUnity.getData();         // "tempUnity" recebe o valor atual escrito no display
    value = Request_temp(tempUnity);               // Atribui o valor de distancia lido no sensor NTC a variável auxiliar
    Temperature.write(value, 2);                   // Escreve no LcmVAr "Temperature" o valor do sensor de acordo a unidade no display
    if (tempUnity == fahrenheitUnity)              // Verifica se a unidade é Fahrenheit
    {
      valueAlarm = ((tempMaxC * 9) / 5.0) + 32;    // Atribui o valor de temperatura máxima a variável auxiliar
      AlarmTempMax.write(valueAlarm);              // Escreve no LcmVAr "AlarmDistMax" o seu valor em polegadas
      valueAlarm = ((tempMinC * 9) / 5.0) + 32;    // Atribui o valor de temperatura mínima a variável auxiliar
      AlarmTempMin.write(valueAlarm);              // Escreve no LcmVAr "AlarmDistMin" o seu valor em polegadas
    }
    else
    {
      AlarmTempMax.write(tempMaxC);                // Escreve no LcmVAr "AlarmTempMax" o seu valor em centímetro
      AlarmTempMin.write(tempMinC);                // Escreve no LcmVAr "AlarmTempMin" o seu valor em centímetro
    }
  }
  // *******************************************************************************************************************************************
  // * Configuração dos valores mínimo e máximo do sensor de distância
  // *
  if (AlarmTempMax.available())                    // Verifica se o valor máximo foi alterado
  {
    long valueAlarm = 0;                           // Variável auxiliar que recebe o valor máximo
    if (tempUnity == fahrenheitUnity)              // Verifica em qual unidade foi escrito o novo valor
    {
      valueAlarm = AlarmTempMax.getData();         // Atribui o valor de distancia máximo a variável auxiliar
      tempMaxC = (valueAlarm - 32) * 5.0 / 9.0;    // "tempMaxC" recebe o novo valor convertido de Fahrenheit para Celsius
    }
    else
    {
      tempMaxC = AlarmTempMax.getData();           // "temp_max_C" recebe o  novo valor máximo em Celsius
    }
  }
  if (AlarmTempMin.available())                    // Verifica em qual unidade foi escrito o novo valor
  {
    long valueAlarm = 0;                           // Variável auxiliar que recebe o valor mínimo
    if (tempUnity == fahrenheitUnity)              // Verifica  o valor "temp_unity"
    {
      valueAlarm = AlarmTempMin.getData();         // Verifica em qual unidade foi escrito o novo valor
      tempMinC = (valueAlarm - 32) * 5.0 / 9.0;    // "tempMinC" recebe o novo valor convertido de Fahrenheit para Celsius
    }
    else
    {
      tempMinC = AlarmTempMin.getData();           // "tempMinC" recebe o  novo valor mínimo em Celsius
    }
  }

  if (alarmtemp || alarmdist)           // Verifica se algum limite excedido
  {
    digitalWrite(buzzerPin, HIGH);      // Liga o Alarme ("BuzzerPin")
  }
  else
  {
    digitalWrite(buzzerPin, LOW);       // Desliga o Alarme ("BuzzerPin").
  }

  // *
  // * Fim do loop
  // *******************************************************************************************************************************************
}

// *******************************************************************************************************************************************
// * Função de requisição  de leitura da temperatura do sensor NTC
// *******************************************************************************************************************************************
float Request_temp(int unity)
{
  float  temperature;                                  // Variáveis temporárias que armazenam as temperaturas em Kelvin, graus Celsius e Fahrenheit, respectivamente
  int sample;                                          // Armazena temporariamente o valor lido na porta analógica
  float vOut = 0;                                      // Armazena temporariamente os valores lido na porta analógica
  for (int i = 0; i < sampling; i++)                   // Faz a coleta de valores do pino analógico do NTC
  {
    sample = analogRead(ntcPin);                       // Atribui o valor lido no "NTC_Pin" a variável "sample"
    vOut += sample;                                    // Armazena todos os valores somados da varável "sample" na variável "vOut"
  }
  vOut /= sampling;                                    // Faz a média dos valores coletados na "vOut" pelo valor de n?mero de amostras "sampling"
  vOut = (analogResolution / vOut) - 1;                // "vOut" recebe o valor de "Analog_Resolution"  dividido por "vOut" menos 1
  vOut = seriesResistor * vOut;                        // "vOut" recebe o valor de "SeriesResistor" vezes "vOut"
  temperature = vOut / ntcValue;                       // "temperature" recebe "vOut" dividido por "NTC_Value"
  temperature = log(temperature);                      // "temperature" recebe o logaritmo  de "temperature"
  temperature /= coefBeta;                             // "temperature"  ?" dividido por "coefBeta"
  temperature += 1.0 / (tempNominal + 273.15);         // "temperature"  ? somada ao inverso da soma "Temp_nominal" e 273,15
  temperature = 1.0 / temperature;                     // "temperature" recebe o inverso de "temperature", resultando na temperatura em Kelvin
  if (unity)                                           //verifica  a variável "unity"
  {
    return ((temperature - 273.15) * 9 / 5.0) + 32;    // se "unity" for verdadeira retorna a "temperature_F"
  }
  else
  {
    return  temperature - 273.15;                      // se "unity" for falsa retorna a "temperature_C"
    
  }
}

// *******************************************************************************************************************************************
// * Função de requisição  de leitura de distância do sensor ultrassónico
// *******************************************************************************************************************************************
float Request_dist(int unity)
{
  long  duration;                        // Armazena o tempo de ECHO
  digitalWrite(trigPin, LOW);            // Escreve "LOW" no pino "TrigPin"
  delayMicroseconds(2);                  // Espera 2 microssegundos
  digitalWrite(trigPin, HIGH);           // Escreve "HIGH" no pino "TrigPin"
  delayMicroseconds(10);                 // Espera 10 microssegundos
  digitalWrite(trigPin, LOW);            // Escreve "LOW" no pino "TrigPin"
  duration = pulseIn(echopPin, HIGH);    // "duration" recebe a duração em que o pino "EchoPin" esteve "HIGH"

  if (unity)
  {
    return duration / 74.0 / 2.0;       // se "unity" for verdadeira retorna a "distance_inch"
  }
  else
  {
    return duration / 29.0 / 2.0;       // se "unity" for falsa retorna a "distance_cm"
  }
}
